@@ -2,6 +2,10 @@
 Tahoe Identity Provider backend.
 """
 
+import logging
+import time
+
+from django.conf import settings
 from social_core.backends.oauth import BaseOAuth2
 
 from .constants import BACKEND_NAME
@@ -15,6 +19,9 @@ from .permissions import (
 )
 
 
+logger = logging.getLogger(__name__)
+
+
 class TahoeIdpOAuth2(BaseOAuth2):
     name = BACKEND_NAME
 
@@ -34,13 +41,21 @@ class TahoeIdpOAuth2(BaseOAuth2):
         """
         Overrides the parent's class `auth_params` to add the organization parameter
         to the auth request.
+
         The API requires us to pass the tenant ID since we are not going
         to ask the user to manually enter their organization name in the login form.
         If we decide against this, we need to enable `Display Organization Prompt` in
         FusionAuth Management Console.
+
+        On the other hand, we can add `idp_hint` parameter to the authorization URL to
+        allow FusionAuth to automatically redirect to the provider's login page instead of
+        showing FusionAuth form with (Login to SAML) button
         """
         params = super().auth_params(state=state)
         params["tenantId"] = helpers.get_tenant_id()
+        default_idp_hint = helpers.get_default_idp_hint()
+        if default_idp_hint:
+            params["idp_hint"] = default_idp_hint
         return params
 
     def get_key_and_secret(self):
@@ -81,11 +96,25 @@ class TahoeIdpOAuth2(BaseOAuth2):
         Fetches the user details from response's JWT and build the social_core JSON object.
         """
         tahoe_idp_uuid = response["userId"]
-        idp_user = helpers.fusionauth_retrieve_user(tahoe_idp_uuid)
+        username = None
+
+        # Deal with race conditions in setting of FusionAuth user username
+        # when not set explicitly by user through a Form.
+        # see https://appsembler.atlassian.net/browse/ENG-80
+        api_retries = 0
+        max_retries = settings.FEATURES.get('TAHOE_MAX_IDP_USER_API_RETRIES', 5)
+        while username is None:
+            if api_retries <= max_retries:
+                idp_user = helpers.fusionauth_retrieve_user(tahoe_idp_uuid)
+                username = idp_user.get("username")
+                time.sleep(1)
+                api_retries += 1
+            else:
+                username = idp_user["id"]
+                logger.warning("tahoe-idp found no username from IdP.  Set to %s", username)
 
         user_data = idp_user.get("data", {})
         user_data_role = get_role_with_default(user_data)
-        username = idp_user.get("username", idp_user["id"])
 
         return {
             "username": username,
